#!/usr/bin/env node

require('isomorphic-fetch')
const rollup = require('rollup').rollup
const uglify = require('uglify-js')
const babel = require('rollup-plugin-babel')
const nodeResolve = require('rollup-plugin-node-resolve')
const commonjs = require('rollup-plugin-commonjs')
const archiver = require('archiver')
const exec = require('child_process').execSync
const fs = require('fs')
const path = require('path')
const deploy = require('../util/qiniu')
const paths = require('../config/path')
const md5File = require('md5-file')
const NODE_ENV = process.env.NODE_ENV
const minimist = require('minimist');
let cache
let config

const argv = minimist(process.argv.slice(2), {
	boolean: [
		// whether to deploy files or not
		"deploy",
	],
	default: {
		deploy: false,
	},
});

const date = new Date()
const dateString = `${date.getFullYear()}${date.getMonth()+1}${date.getDate()}${date.getHours()}${date.getMinutes()}${date.getSeconds()}`
process.env.BUILD_VERSION = dateString

if (argv.deploy) {
  config = require('../config/rollup.pro')
} else {
  config = Object.assign({} , require('../config/rollup.dev.js'), {
    cache: cache
  })
}

let packageBundleName
const appConfig = require(paths.appEnvoyConfig)
try {
  if (argv.deploy) {
    if (!appConfig.name) {
      console.log(red('No appConfig name provided in envoy.config.js'))
      return
    }
    if (!appConfig.engineVersion) {
      console.log(red('No engineVersion provided in envoy.config.js'))
      return
    }
    if (!appConfig.accessKey) {
      console.log(red('No accessKey provided in envoy.config.js'))
      return
    }
    if (!appConfig.privateKey) {
      console.log(red('No privateKey provided in envoy.config.js'))
      return
    }
    packageBundleName = `${appConfig.name.replace(' ', '_')}_${appConfig.engineVersion.replace('.', '_')}_${process.env.BUILD_VERSION}`
  } else {
    packageBundleName = appConfig.name || 'test'
  }
} catch (error) {
  packageBundleName = 'test'
  
}
const check = require('../util/checkVersion')

build()

function build() {
  exec(`rm -rf ${paths.appDist} && mkdir ${paths.appDist}`)
  exec(`rm -rf ${paths.appZip} && mkdir ${paths.appZip}`)
  exec(`cp -r ${paths.appResource} ${paths.appDistResource}`)
  
  rollup(config).then(bundle => {
    cache = bundle
    const code = bundle.generate({
      format: 'iife',
      useStrict: false,
      moduleName: 'App',
    }).code
  
    if (argv.deploy) {
      const minified = uglify.minify(code, {
        fromString: true,
        output: {
          ascii_only: true
        }
      }).code
      return write(paths.appOutput, minified)
    } else {
      return write(paths.appOutput, code)
    }
  }).then(() => {
      
    let output = fs.createWriteStream(path.join(paths.appZip, `${packageBundleName}.zip`))
    const archive = archiver('zip')
    archive.on('error', e => {
      throw err
    })
    archive.pipe(output)
    archive.directory(paths.appDist, packageBundleName)
    return archive.finalize()
  }).then(() => {
    return check()
  }).then(isOutOfDate => {
    if (isOutOfDate) {
      const str = `You are using Envoy v${isOutOfDate.local}, v${isOutOfDate.remote} is available`
      console.log(blue(str))
    }
    if (argv.deploy) {
      const filePath = path.join(paths.appZip, `${packageBundleName}.zip`)
      const bucket = 'mesh-package'
      const key = `${packageBundleName}.zip`
      const fingerprint = md5File.sync(filePath)
      return deploy(filePath, bucket, key, appConfig, fingerprint)
    }
  }).then(data => {
    if (data) {
      console.log(blue(data))
    }
  }).catch(e => {
    console.log(red(e))
  })
}


function write (dest, code) {
  return new Promise(function (resolve, reject) {
    fs.writeFile(dest, code, function (err) {
      if (err) return reject(err)
      console.log(blue(dest) + ' ' + getSize(code))
      resolve()
    })
  })
}

function getSize (code) {
  return (code.length / 1024).toFixed(2) + 'kb'
}

function blue (str) {
  return '\x1b[1m\x1b[34m' + str + '\x1b[39m\x1b[22m'
}

function red(str) {
  return "\x1b[31m\x1b[1m" + str + "\x1b[0m"
}
